/* NitroShare - A simple network file sharing tool.
   Copyright (C) 2012 Nathan Osman

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <QDateTime>
#include <QDebug>
#include <QMutableMapIterator>
#include <QNetworkAddressEntry>

#include <discovery/CBroadcastServer.h>
#include <util/network.h>
#include <util/settings.h>

CBroadcastServer::CBroadcastServer()
    : m_start_time(QDateTime::currentMSecsSinceEpoch())
{
    connect(this, SIGNAL(readyRead()), SLOT(OnMessage()));
    connect(&m_timer, SIGNAL(timeout()), SLOT(OnTimer()));
}

void CBroadcastServer::Init()
{
    // Stop the timer and stop listening for incoming connections
    close();
    m_timer.stop();

    // Attempt to find the broadcast address on the default address
    if(!Network::FindIPv4Address(Network::GetCurrentInterface(), Network::BroadcastAddress, &m_broadcast_address))
    {
        if(Settings::Notify("Error"))
            emit Message("Unable to find an IPv4 broadcast address for the current interface.", QSystemTrayIcon::Critical);

        return;
    }

    bind(Settings::Get("Network/BroadcastPort").toUInt(), QUdpSocket::ShareAddress);

    // Start the timer and send out an initial ping
    m_timer.setInterval(Settings::Get("Network/BroadcastInterval").toInt());
    m_timer.start();
    OnTimer();
}

void CBroadcastServer::OnMessage()
{
    // We've received a datagram from one of the other machines
    // Begin by allocating enough room for the datagram
    QByteArray message;
    message.resize(pendingDatagramSize());

    // Now read the datagram, grabbing the host address in the process
    QHostAddress src;
    readDatagram(message.data(), message.size(), &src);

    // Split the message into its two parts
    QList<QByteArray> parts = message.split('\n');
    if(parts.size() == 3)
    {
        // Make sure we aren't the sender
        if(Settings::Get("General/MachineName").toString() != parts[0])
            UpdateMachineEntry(parts[0], src, parts[1].toUShort(), parts[2].toLongLong());
    }
    else
        // Silently drop the packet and print a debug message
        qDebug() << "Packet received from" << src.toString() << "is corrupt.";
}

void CBroadcastServer::OnTimer()
{
    // Write the ping datagram
    writeDatagram(QByteArray(tr("%1\n%2\n%3").arg(Settings::Get("General/MachineName").toString())
                                             .arg(Settings::Get("Network/TransmissionPort").toUInt())
                                             .arg(m_start_time).toUtf8()),
                  m_broadcast_address, Settings::Get("Network/BroadcastPort").toUInt());

    // Check to see if any PCs have left the local network
    qint64 current_time = QDateTime::currentMSecsSinceEpoch();

    QMutableMapIterator<QByteArray, CMachine> i(m_machines);
    while(i.hasNext())
    {
        i.next();
        if(i.value().last_ping + (Settings::Get("Network/TimeoutInterval").toInt() * 1000) < current_time)
        {
            // Perform the notification if necessary
            if(Settings::Notify("PCQuit"))
                emit Message(tr("%1 has left the network.").arg(QString(i.value().name)),
                             QSystemTrayIcon::Information);

            // Perform the removal now
            m_machines.remove(i.key());
        }
    }
}

void CBroadcastServer::UpdateMachineEntry(QByteArray hostname, QHostAddress src,
                                          quint16 port, qint64 start_time)
{
    // Add this machine to the table if it isn't already there
    if(!m_machines.contains(hostname))
    {
        CMachine machine;
        machine.name = hostname;

        m_machines.insert(hostname, machine);

        // If the setting has been set and this truly is a new user,
        // notify the user that a machine has joined the group
        if(start_time > m_start_time && Settings::Notify("NewPC"))
            emit Message(tr("%1 has joined the network.").arg(QString(hostname)), QSystemTrayIcon::Information);
    }

    // Now update the machine
    m_machines[hostname].address   = src;
    m_machines[hostname].port      = port;
    m_machines[hostname].last_ping = QDateTime::currentMSecsSinceEpoch();
}
