/* NitroShare - A simple network file sharing tool.
   Copyright (C) 2012 Nathan Osman

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <QDir>
#include <QFile>
#include <QRegExp>
#include <QVariantMap>
#include <qjson/qobjecthelper.h>
#include <qjson/parser.h>

#include <dialogs/CAcceptPromptDialog.h>
#include <file/CFileReceiver.h>
#include <util/defaults.h>
#include <util/settings.h>

CFileReceiver::CFileReceiver(QObject * parent, int handle)
    : CBasicSocket(parent), m_state(WaitingForHeader)
{
    connect(this, SIGNAL(Data(QByteArray)), SLOT(OnData(QByteArray)));

    setSocketDescriptor(handle);
}

CFileReceiver::~CFileReceiver()
{
    qDeleteAll(m_headers);
}

void CFileReceiver::OnData(QByteArray data)
{
    // The action we perform depends on what state we are in
    if(m_state == WaitingForHeader)
    {
        // Parse the JSON response
        QVariantMap map = ParseJSON(data);

        ParseHeaders(map);
    }
    else
    {
        // If we are expecting another file, read it
        if(m_headers.size())
            SaveFile(m_headers.takeFirst(), data);

        // If we're done, then close the socket!
        if(!m_headers.size())
            disconnectFromHost();
    }
}

void CFileReceiver::ParseHeaders(QVariantMap map)
{
    // Perform a basic sanity check
    if(!map.contains("hostname") || !map.contains("files"))
    {
        emit Message("Unable to parse the JSON response sent from the client.",
                     QSystemTrayIcon::Critical);

        disconnectFromHost();
    }

    // Process the results
    m_hostname = map["hostname"].toString();
    foreach(QVariant variant, map["files"].toList())
    {
        CFileHeader * header = new CFileHeader;
        QJson::QObjectHelper::qvariant2qobject(variant.toMap(), header);
        m_headers.append(header);
    }

    // Make sure the files pass the current policy
    if(TestAgainstPolicy())
    {
        // We are now waiting for the first file to arrive
        m_state = WaitingForTransfer;
        SendStatus("continue");
    }
    else
        disconnectFromHost();
}

bool CFileReceiver::TestAgainstPolicy()
{
    int policy = Settings::Get("Security/IncomingPolicy").toInt();

    // Let's get the two easy ones out of the way
    if(policy == Defaults::AcceptAll)  return true;
    if(policy == Defaults::AcceptNone) return false;

    // Next check if we need to prompt the user
    if(policy == Defaults::AcceptPrompt)
    {
        CAcceptPromptDialog dialog(m_hostname, m_headers);
        return dialog.exec();
    }

    // Note: we don't perform the RegEx match until we are actually
    // ready to perform the transfer since we still need the headers
    // for files we do not wish to transfer.

    return true;
}

void CFileReceiver::SaveFile(CFileHeader * header, QByteArray data)
{
    // Check the security policy to see if we need to filter
    // incoming filenames.

    if(Settings::Get("Security/IncomingPolicy").toInt() == Defaults::AcceptPattern)
    {
        // Perform the regular expression match
        QRegExp pattern(Settings::Get("Security/IncomingPattern").toString());
        if(pattern.indexIn(header->GetFilename()) == -1)
            return;
    }

    // We're good - so now we simply create the file and dump the contents into it
    QDir storage_dir(Settings::Get("General/ReceivedFilesDirectory").toString());
    QFile file(storage_dir.absoluteFilePath(header->GetFilename()));

    if(!file.open(QIODevice::WriteOnly))
    {
        if(Settings::Notify("Error"))
            emit Message(tr("Unable to store received file '%1'.").arg(header->GetFilename()),
                         QSystemTrayIcon::Critical);

        // Skip ahead to the next one
        return;
    }

    file.write(data);
}
