/* NitroShare - A simple network file sharing tool.
   Copyright (C) 2012 Nathan Osman

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>. */

#include <QFile>
#include <QFileInfo>
#include <QVariantList>
#include <qjson/qobjecthelper.h>
#include <qjson/serializer.h>

#include <file/CFileHeader.h>
#include <file/CFileSender.h>

CFileSender::CFileSender(QObject * parent, QString hostname, QStringList filenames)
    : CBasicSocket(parent), m_hostname(hostname), m_filenames(filenames),
      m_state(WaitingForConnection)
{
    connect(this, SIGNAL(connected()),      SLOT(OnConnected()));
    connect(this, SIGNAL(Data(QByteArray)), SLOT(OnData(QByteArray)));
}

void CFileSender::Start(CMachine machine)
{
    // Attempt a connection to the host
    connectToHost(machine.address, machine.port);
}

void CFileSender::OnConnected()
{
    // Prepare the file headers
    QVariantList header_list;

    foreach(QString filename, m_filenames)
    {
        QFileInfo info(filename);

        CFileHeader header;
        header.SetFilename(info.fileName());
        header.SetUncompressedSize(info.size());

        header_list.append(QJson::QObjectHelper::qobject2qvariant(&header));
    }

    // Prepare the final structure
    QVariantMap map;
    map["hostname"] = m_hostname;
    map["files"]    = header_list;

    // Convert it to JSON and send it over the wire
    QJson::Serializer serializer;
    SendData(serializer.serialize(map));

    // Now we are in the state where we are waiting for the response
    // from the user for the files we have sent
    m_state = WaitingForResponse;
}

void CFileSender::OnData(QByteArray data)
{
    // Make sure the response contains "status"
    QVariantMap response = ParseJSON(data);

    if(!response.contains("status"))
    {
        emit Message("Unable to parse response from server.", QSystemTrayIcon::Critical);
        disconnectFromHost();
        return;
    }

    // If we're waiting for the server's initial response, then
    // we check to see if we should continue or not.
    if(m_state == WaitingForResponse && response["status"] == "continue")
        SendFiles();

    // We're done regardless
    disconnectFromHost();
}

void CFileSender::SendFiles()
{
    // Open each file and write it to the socket
    foreach(QString filename, m_filenames)
    {
        QFile file(filename);

        if(file.open(QIODevice::ReadOnly))
            SendData(file.readAll());
    }
}
